==> waffle_hub-0.2.16.tar.gz: waffle_hub-0.2.16/waffle_hub/hub/adapter/autocare_dlt/autocare_dlt_hub.py <==
"""
Tx Model Hub
See Hub documentation for more details about usage.
"""

import warnings
from pathlib import Path
from typing import Union

import autocare_dlt
import tbparse
import torch
from autocare_dlt.core.model import build_model
from autocare_dlt.tools import train
from box import Box
from torchvision import transforms as T
from waffle_utils.file import io
from waffle_utils.utils import type_validator

from waffle_hub import DataType, TaskType
from waffle_hub.hub import Hub
from waffle_hub.hub.adapter.autocare_dlt.configs import (
    get_data_config,
    get_model_config,
)
from waffle_hub.hub.model.wrapper import ModelWrapper
from waffle_hub.schema.configs import TrainConfig
from waffle_hub.utils.working_info_logger import TrainingInfoLogger

from .config import DATA_TYPE_MAP, DEFAULT_PARAMS, MODEL_TYPES, WEIGHT_PATH


class AutocareDLTHub(Hub):
    BACKEND_NAME = DataType.AUTOCARE_DLT
    MODEL_TYPES = MODEL_TYPES
    MULTI_GPU_TRAIN = False
    DEFAULT_PARAMS = DEFAULT_PARAMS
    DEFAULT_ADVANCE_PARAMS = {}

    DATA_TYPE_MAP = DATA_TYPE_MAP
    WEIGHT_PATH = WEIGHT_PATH

    def __init__(
        self,
        name: str,
        task: str = None,
        model_type: str = None,
        model_size: str = None,
        categories: Union[list[dict], list] = None,
        root_dir: str = None,
        backend: str = None,
        version: str = None,
        *args,
        **kwargs,
    ):
        if backend is not None and AutocareDLTHub.BACKEND_NAME != backend:
            raise ValueError(
                f"Backend {backend} is not supported. Please use {AutocareDLTHub.BACKEND_NAME}"
            )

        if version is not None and autocare_dlt.__version__ != version:
            warnings.warn(
                f"You've loaded the Hub created with autocare_dlt=={version}, \n"
                + f"but the installed version is {autocare_dlt.__version__}."
            )

        super().__init__(
            name=name,
            backend=AutocareDLTHub.BACKEND_NAME,
            version=autocare_dlt.__version__,
            task=task,
            model_type=model_type,
            model_size=model_size,
            categories=categories,
            root_dir=root_dir,
        )

        self.model_json_output_path = self.hub_dir / self.CONFIG_DIR / "model.json"

    @classmethod
    def new(
        cls,
        name: str,
        task: str = None,
        model_type: str = None,
        model_size: str = None,
        categories: Union[list[dict], list] = None,
        root_dir: str = None,
        *args,
        **kwargs,
    ):
        """Create Tx Model Hub.

        Args:
            name (str): Hub name
            task (str, optional): Task Name. See UltralyticsHub.TASKS. Defaults to None.
            model_type (str, optional): Model Type. See UltralyticsHub.MODEL_TYPES. Defaults to None.
            model_size (str, optional): Model Size. See UltralyticsHub.MODEL_SIZES. Defaults to None.
            categories (Union[list[dict], list]): class dictionary or list. [{"supercategory": "name"}, ] or ["name",].
            root_dir (str, optional): Root directory of hub repository. Defaults to None.
        """

        warnings.warn("UltralyticsHub.new() is deprecated. Please use Hub.new() instead.")

        return cls(
            name=name,
            task=task,
            model_type=model_type,
            model_size=model_size,
            categories=categories,
            root_dir=root_dir,
        )

    # Hub Utils
    def get_preprocess(self, *args, **kwargs):

        if self.task == TaskType.OBJECT_DETECTION:
            normalize = T.Normalize([0, 0, 0], [1, 1, 1], inplace=True)

            def preprocess(x, *args, **kwargs):
                return normalize(x)

        elif self.task == TaskType.CLASSIFICATION:
            normalize = T.Normalize([0, 0, 0], [1, 1, 1], inplace=True)

            def preprocess(x, *args, **kwargs):
                return normalize(x)

        elif self.task == TaskType.TEXT_RECOGNITION:
            normalize = T.Normalize([0, 0, 0], [1, 1, 1], inplace=True)

            def preprocess(x, *args, **kwargs):
                return normalize(x)

        elif self.task == TaskType.SEMANTIC_SEGMENTATION:
            normalize = T.Normalize([0], [1], inplace=True)
            gray_sacle = T.Grayscale()

            def preprocess(x, *args, **kwargs):
                return normalize(gray_sacle(x))

        else:
            raise NotImplementedError(f"task {self.task} is not supported yet")

        return preprocess

    def get_postprocess(self, *args, **kwargs):

        if self.task == TaskType.OBJECT_DETECTION:

            def inner(x: torch.Tensor, *args, **kwargs):
                xyxy = x[0]
                scores = x[1]
                class_ids = x[2]

                return xyxy, scores, class_ids

        elif self.task == TaskType.CLASSIFICATION:

            def inner(x: torch.Tensor, *args, **kwargs):
                x = [t.squeeze(-1).squeeze(-1) for t in x]
                return x

        elif self.task == TaskType.TEXT_RECOGNITION:

            def inner(x: torch.Tensor, *args, **kwargs):
                scores, character_class_ids = x.max(dim=-1)
                return character_class_ids, scores

        elif self.task == TaskType.SEMANTIC_SEGMENTATION:

            def inner(x: torch.Tensor, *args, **kwargs):
                return x

        else:
            raise NotImplementedError(f"task {self.task} is not supported yet")

        return inner

    def get_metrics(self) -> list[dict]:
        """Get metrics from tensorboard log directory.
        Args:
            tbdir (Union[str, Path]): tensorboard log directory
        Returns:
            list[dict]: list of metrics
        """
        tb_log_dir = self.artifact_dir / "train" / "tensorboard"
        if not tb_log_dir.exists():
            return []

        sr = tbparse.SummaryReader(tb_log_dir)
        df = sr.scalars

        # Sort the data frame by step.
        # Make a list of dictionaries of tag and value.
        if df.empty:
            return []

        metrics = (
            df.sort_values("step")
            .groupby("step")
            .apply(lambda x: [{"tag": s, "value": v} for s, v in zip(x.tag, x.value)])
            .to_list()
        )

        return metrics

    # Train Hook
    def on_train_start(self, cfg: TrainConfig):
        # set data
        cfg.dataset_path: Path = Path(cfg.dataset_path)
        train_coco_file = cfg.dataset_path / "train.json"
        val_coco_file = cfg.dataset_path / "val.json"
        test_coco_file = cfg.dataset_path / "test.json"
        data_config = get_data_config(
            self.DATA_TYPE_MAP[self.task],
            cfg.image_size if isinstance(cfg.image_size, list) else [cfg.image_size, cfg.image_size],
            cfg.batch_size,
            cfg.workers,
            str(train_coco_file),
            str(cfg.dataset_path / "images"),
            str(val_coco_file),
            str(cfg.dataset_path / "images"),
            str(test_coco_file) if test_coco_file.exists() else str(val_coco_file),
            str(cfg.dataset_path / "images"),
        )
        if self.model_type == "LicencePlateRecognition":
            data_config["data"]["mode"] = "lpr"
        if self.model_type == "Segmenter":
            data_config["data"]["gray"] = True

        cfg.data_config = self.artifact_dir / "data.json"
        io.save_json(data_config, cfg.data_config, create_directory=True)

        if self.task == TaskType.CLASSIFICATION:
            super_cat = [[c.supercategory, c.name] for c in self.categories]
            super_cat_dict = {}
            for super_cat, cat in super_cat:
                if super_cat not in super_cat_dict:
                    super_cat_dict[super_cat] = []
                super_cat_dict[super_cat].append(cat)

            categories = []
            for super_cat, cat in super_cat_dict.items():
                categories.append({super_cat: cat})

        else:
            categories = self.get_category_names()

        model_config = get_model_config(
            self.model_type,
            self.model_size,
            categories,
            cfg.seed,
            cfg.learning_rate,
            cfg.letter_box,
            cfg.epochs,
        )

        cfg.model_config = self.artifact_dir / "model.json"
        io.save_json(model_config, cfg.model_config, create_directory=True)

        # pretrained model
        cfg.pretrained_model = (
            cfg.pretrained_model
            if cfg.pretrained_model is not None
            else self.WEIGHT_PATH[self.task][self.model_type][self.model_size]
        )
        if not Path(cfg.pretrained_model).exists():
            cfg.pretrained_model = None
            warnings.warn(f"{cfg.pretrained_model} does not exists. Train from scratch.")

        cfg.dataset_path = str(cfg.dataset_path.absolute())

    def training(self, cfg: TrainConfig, info_logger: TrainingInfoLogger):
        results = train.run(
            exp_name="train",
            model_cfg=str(cfg.model_config),
            data_cfg=str(cfg.data_config),
            gpus="-1" if cfg.device == "cpu" else str(cfg.device),
            output_dir=str(self.artifact_dir),
            ckpt=cfg.pretrained_model,
            overwrite=True,
        )
        if results is None:
            raise RuntimeError("Training failed")
        del results

    def on_train_end(self, cfg: TrainConfig):
        best_ckpt_path = self.artifact_dir / "train" / "best_ckpt.pth"
        last_epoch_ckpt_path = self.artifact_dir / "train" / "last_epoch_ckpt.pth"
        model_json_path = self.artifact_dir / "model.json"

        if best_ckpt_path.exists():
            io.copy_file(best_ckpt_path, self.best_ckpt_file, create_directory=True)
        if last_epoch_ckpt_path.exists():
            io.copy_file(last_epoch_ckpt_path, self.last_ckpt_file, create_directory=True)
        if model_json_path.exists():
            io.copy_file(model_json_path, self.model_json_output_path, create_directory=True)

        metrics = self.get_metrics()
        if metrics:
            io.save_json(metrics, self.metric_file, create_directory=True)

    # Inference Hook
    def get_model(self):
        """Get model.
        Returns:
            ModelWrapper: Model wrapper
        """
        self.check_train_sanity()

        # get adapt functions
        preprocess = self.get_preprocess()
        postprocess = self.get_postprocess()

        # get model
        categories = [x["name"] for x in self.categories]
        cfg = io.load_json(self.model_json_output_path)
        cfg["ckpt"] = str(self.best_ckpt_file)
        if self.task == TaskType.TEXT_RECOGNITION:
            cfg["model"]["Prediction"]["num_classes"] = len(categories) + 1
        else:
            cfg["model"]["head"]["num_classes"] = len(categories)
        cfg["num_classes"] = len(categories)
        model, categories = build_model(Box(cfg), strict=True)

        model = ModelWrapper(
            model=model.eval(),
            preprocess=preprocess,
            postprocess=postprocess,
        )

        return model

==> waffle_hub-0.3.0.tar.gz: waffle_hub-0.3.0/waffle_hub/hub/adapter/autocare_dlt/autocare_dlt_hub.py <==
"""
Tx Model Hub
See Hub documentation for more details about usage.
"""

import warnings
from pathlib import Path
from typing import Union

import autocare_dlt
import tbparse
import torch
from autocare_dlt.core.model import build_model
from autocare_dlt.tools import train
from box import Box
from torchvision import transforms as T
from waffle_utils.file import io
from waffle_utils.utils import type_validator

from waffle_hub import DataType, TaskType
from waffle_hub.hub import Hub
from waffle_hub.hub.adapter.autocare_dlt.configs import (
    get_data_config,
    get_model_config,
)
from waffle_hub.hub.model.wrapper import ModelWrapper
from waffle_hub.schema.configs import TrainConfig
from waffle_hub.utils.running_status_logger import TrainingStatusLogger

from .config import DATA_TYPE_MAP, DEFAULT_PARAMS, MODEL_TYPES, WEIGHT_PATH


class AutocareDLTHub(Hub):
    BACKEND_NAME = DataType.AUTOCARE_DLT
    MODEL_TYPES = MODEL_TYPES
    MULTI_GPU_TRAIN = False
    DEFAULT_PARAMS = DEFAULT_PARAMS
    DEFAULT_ADVANCE_PARAMS = {}

    DATA_TYPE_MAP = DATA_TYPE_MAP
    WEIGHT_PATH = WEIGHT_PATH

    def __init__(
        self,
        name: str,
        task: str = None,
        model_type: str = None,
        model_size: str = None,
        categories: Union[list[dict], list] = None,
        root_dir: str = None,
        backend: str = None,
        version: str = None,
        *args,
        **kwargs,
    ):
        if backend is not None and AutocareDLTHub.BACKEND_NAME != backend:
            raise ValueError(
                f"Backend {backend} is not supported. Please use {AutocareDLTHub.BACKEND_NAME}"
            )

        if version is not None and autocare_dlt.__version__ != version:
            warnings.warn(
                f"You've loaded the Hub created with autocare_dlt=={version}, \n"
                + f"but the installed version is {autocare_dlt.__version__}."
            )

        super().__init__(
            name=name,
            backend=AutocareDLTHub.BACKEND_NAME,
            version=autocare_dlt.__version__,
            task=task,
            model_type=model_type,
            model_size=model_size,
            categories=categories,
            root_dir=root_dir,
        )

        self.model_json_output_path = self.hub_dir / self.CONFIG_DIR / "model.json"

    @classmethod
    def new(
        cls,
        name: str,
        task: str = None,
        model_type: str = None,
        model_size: str = None,
        categories: Union[list[dict], list] = None,
        root_dir: str = None,
        *args,
        **kwargs,
    ):
        """Create Tx Model Hub.

        Args:
            name (str): Hub name
            task (str, optional): Task Name. See UltralyticsHub.TASKS. Defaults to None.
            model_type (str, optional): Model Type. See UltralyticsHub.MODEL_TYPES. Defaults to None.
            model_size (str, optional): Model Size. See UltralyticsHub.MODEL_SIZES. Defaults to None.
            categories (Union[list[dict], list]): class dictionary or list. [{"supercategory": "name"}, ] or ["name",].
            root_dir (str, optional): Root directory of hub repository. Defaults to None.
        """

        warnings.warn("UltralyticsHub.new() is deprecated. Please use Hub.new() instead.")

        return cls(
            name=name,
            task=task,
            model_type=model_type,
            model_size=model_size,
            categories=categories,
            root_dir=root_dir,
        )

    # Hub Utils
    def get_preprocess(self, *args, **kwargs):

        if self.task == TaskType.OBJECT_DETECTION:
            normalize = T.Normalize([0, 0, 0], [1, 1, 1], inplace=True)

            def preprocess(x, *args, **kwargs):
                return normalize(x)

        elif self.task == TaskType.CLASSIFICATION:
            normalize = T.Normalize([0, 0, 0], [1, 1, 1], inplace=True)

            def preprocess(x, *args, **kwargs):
                return normalize(x)

        elif self.task == TaskType.TEXT_RECOGNITION:
            normalize = T.Normalize([0, 0, 0], [1, 1, 1], inplace=True)

            def preprocess(x, *args, **kwargs):
                return normalize(x)

        elif self.task == TaskType.SEMANTIC_SEGMENTATION:
            normalize = T.Normalize([0], [1], inplace=True)
            gray_sacle = T.Grayscale()

            def preprocess(x, *args, **kwargs):
                return normalize(gray_sacle(x))

        else:
            raise NotImplementedError(f"task {self.task} is not supported yet")

        return preprocess

    def get_postprocess(self, *args, **kwargs):

        if self.task == TaskType.OBJECT_DETECTION:

            def inner(x: torch.Tensor, *args, **kwargs):
                xyxy = x[0]
                scores = x[1]
                class_ids = x[2]

                return xyxy, scores, class_ids

        elif self.task == TaskType.CLASSIFICATION:

            def inner(x: torch.Tensor, *args, **kwargs):
                x = [t.squeeze(-1).squeeze(-1) for t in x]
                return x

        elif self.task == TaskType.TEXT_RECOGNITION:

            def inner(x: torch.Tensor, *args, **kwargs):
                scores, character_class_ids = x.max(dim=-1)
                return character_class_ids, scores

        elif self.task == TaskType.SEMANTIC_SEGMENTATION:

            def inner(x: torch.Tensor, *args, **kwargs):
                return x

        else:
            raise NotImplementedError(f"task {self.task} is not supported yet")

        return inner

    def get_metrics(self) -> list[dict]:
        """Get metrics from tensorboard log directory.
        Args:
            tbdir (Union[str, Path]): tensorboard log directory
        Returns:
            list[dict]: list of metrics
        """
        tb_log_dir = self.artifact_dir / "train" / "tensorboard"
        if not tb_log_dir.exists():
            return []

        sr = tbparse.SummaryReader(tb_log_dir)
        df = sr.scalars

        # Sort the data frame by step.
        # Make a list of dictionaries of tag and value.
        if df.empty:
            return []

        metrics = (
            df.sort_values("step")
            .groupby("step")
            .apply(lambda x: [{"tag": s, "value": v} for s, v in zip(x.tag, x.value)])
            .to_list()
        )

        return metrics

    # Train Hook
    def on_train_start(self, cfg: TrainConfig):
        # set data
        cfg.dataset_path: Path = Path(cfg.dataset_path)
        train_coco_file = cfg.dataset_path / "train.json"
        val_coco_file = cfg.dataset_path / "val.json"
        test_coco_file = cfg.dataset_path / "test.json"
        data_config = get_data_config(
            self.DATA_TYPE_MAP[self.task],
            cfg.image_size if isinstance(cfg.image_size, list) else [cfg.image_size, cfg.image_size],
            cfg.batch_size,
            cfg.workers,
            str(train_coco_file),
            str(cfg.dataset_path / "images"),
            str(val_coco_file),
            str(cfg.dataset_path / "images"),
            str(test_coco_file) if test_coco_file.exists() else str(val_coco_file),
            str(cfg.dataset_path / "images"),
        )
        if self.model_type == "LicencePlateRecognition":
            data_config["data"]["mode"] = "lpr"
        if self.model_type == "Segmenter":
            data_config["data"]["gray"] = True

        cfg.data_config = self.artifact_dir / "data.json"
        io.save_json(data_config, cfg.data_config, create_directory=True)

        if self.task == TaskType.CLASSIFICATION:
            super_cat = [[c.supercategory, c.name] for c in self.categories]
            super_cat_dict = {}
            for super_cat, cat in super_cat:
                if super_cat not in super_cat_dict:
                    super_cat_dict[super_cat] = []
                super_cat_dict[super_cat].append(cat)

            categories = []
            for super_cat, cat in super_cat_dict.items():
                categories.append({super_cat: cat})

        else:
            categories = self.get_category_names()

        model_config = get_model_config(
            self.model_type,
            self.model_size,
            categories,
            cfg.seed,
            cfg.learning_rate,
            cfg.letter_box,
            cfg.epochs,
        )

        cfg.model_config = self.artifact_dir / "model.json"
        io.save_json(model_config, cfg.model_config, create_directory=True)

        # pretrained model
        cfg.pretrained_model = (
            cfg.pretrained_model
            if cfg.pretrained_model is not None
            else self.WEIGHT_PATH[self.task][self.model_type][self.model_size]
        )
        if not Path(cfg.pretrained_model).exists():
            cfg.pretrained_model = None
            warnings.warn(f"{cfg.pretrained_model} does not exists. Train from scratch.")

        cfg.dataset_path = str(cfg.dataset_path.absolute())

    def training(self, cfg: TrainConfig, status_logger: TrainingStatusLogger):
        results = train.run(
            exp_name="train",
            model_cfg=str(cfg.model_config),
            data_cfg=str(cfg.data_config),
            gpus="-1" if cfg.device == "cpu" else str(cfg.device),
            output_dir=str(self.artifact_dir),
            ckpt=cfg.pretrained_model,
            overwrite=True,
        )
        if results is None:
            raise RuntimeError("Training failed")
        del results

    def on_train_end(self, cfg: TrainConfig):
        best_ckpt_path = self.artifact_dir / "train" / "best_ckpt.pth"
        last_epoch_ckpt_path = self.artifact_dir / "train" / "last_epoch_ckpt.pth"
        model_json_path = self.artifact_dir / "model.json"

        if best_ckpt_path.exists():
            io.copy_file(best_ckpt_path, self.best_ckpt_file, create_directory=True)
        if last_epoch_ckpt_path.exists():
            io.copy_file(last_epoch_ckpt_path, self.last_ckpt_file, create_directory=True)
        if model_json_path.exists():
            io.copy_file(model_json_path, self.model_json_output_path, create_directory=True)

        metrics = self.get_metrics()
        if metrics:
            io.save_json(metrics, self.metric_file, create_directory=True)

    # Inference Hook
    def get_model(self):
        """Get model.
        Returns:
            ModelWrapper: Model wrapper
        """
        self.check_train_sanity()

        # get adapt functions
        preprocess = self.get_preprocess()
        postprocess = self.get_postprocess()

        # get model
        categories = [x["name"] for x in self.categories]
        cfg = io.load_json(self.model_json_output_path)
        cfg["ckpt"] = str(self.best_ckpt_file)
        if self.task == TaskType.TEXT_RECOGNITION:
            cfg["model"]["Prediction"]["num_classes"] = len(categories) + 1
        else:
            cfg["model"]["head"]["num_classes"] = len(categories)
        cfg["num_classes"] = len(categories)
        model, categories = build_model(Box(cfg), strict=True)

        model = ModelWrapper(
            model=model.eval(),
            preprocess=preprocess,
            postprocess=postprocess,
        )

        return model
